#!/usr/bin/perl
use strict;
use File::Basename;
use Fcntl;
use File::ExtAttr ':all';
use IO::File;
use strict;
use Cwd;
use File::Copy;

my $debug = 0;
sub qprint{return if ($ENV{QUIET});     print STDERR @_;}
sub qprintf{return if ($ENV{QUIET});     printf STDERR @_;}
my $debug = $ENV{DEBUG};
sub dprint{return if (!$debug);     print @_;}
sub dprint1{return if ($debug < 1); print STDERR @_;}
sub dprint2{return if ($debug < 2); print @_;}
sub dprint3{return if ($debug < 3); print @_;}
sub dprintf{return if (!$debug); printf STDERR @_;}
sub dprintf2{return if ($debug < 2); printf STDERR @_;}

sub cat_file
{
	my $filename = shift;;
	my $contents;
	my $line;
	open FILE, "< $filename";
	while ($line = <FILE>) {
		$contents .= $line;
	}
	close FILE;
	return $contents;
}

sub parent_dirs
{
        (my $dir) = @_;
        my @res;
        # just like dirname(1), if the arg ends in /
        # then count it.
        if ($dir =~ s/\/$//) {
                push @res, $dir;
        }
        while (length($dir) && $dir ne "/" && $dir ne ".") {
                $dir = dirname($dir);
                push @res, $dir;
        }
        return @res;
}
sub mkdir_or_die
{
	my $dir = shift;
	my $res = 1;
	if (! -d $dir) {
		if (!($res = mkdir $dir)) {
			die "error: $! '$dir'";
		}
	}
	return $res;
}

sub mkdir_p
{
	my $dir = shift;
	my $pdir;
	if ($dir !~ /\/$/) {
		$dir .= "/";
	}
	my @pdirs = reverse(parent_dirs($dir));
	for $pdir (@pdirs) {
		mkdir_or_die($pdir);
	}
}

sub dirpart
{
	my $foo = shift;
	$foo =~ s/(.*\/).*/$1/;
	return $foo;
}

sub file_has_line
{
	my $filename = shift;
	my $pattern = shift;

	open(FILE, "< $filename");
	while (my $line = <FILE>) {
		chomp $line;
		if ($line eq $pattern) {
			return 1;
		}
	}
	close FILE;
	return 0;
}
sub file_append_line
{
	my $filename = shift;
	my $line = shift;

	open(FILE, ">> $filename");
	print FILE $line."\n";
	close FILE;
}
sub file_append_null_string
{
	my $filename = shift;
	my $line = shift;

	open(FILE, ">> $filename");
	print FILE $line."\0";
	close FILE;
}
sub file_read_one_line
{
	my $filename = shift;
	open(FILE, "< $filename");
	my $line = <FILE>;
	chomp $line;
	return $line;
}
sub file_shift_line
{
	my $filename = shift;
	my $tmpfile = $filename.".tmp";
	open(FILE, "< $filename");
	open(TMPFILE, "> $tmpfile");
	my $line = <FILE>;
	while ($line = <FILE>) {
		print TMPFILE $line;
	}
	close FILE;
	close TMPFILE;
	rename $tmpfile, $filename;
	return $line;
}

sub cat_file
{
        my $filename = shift;;
        my $contents;
        my $line;
        open FILE, "< $filename";
        while ($line = <FILE>) {
                $contents .= $line;
        }
        close FILE;
        return $contents;
}
sub append_to_file
{
        my $stuff = shift;
        my $file = shift;
        open FILE, ">> $file";
        print FILE $stuff;
        close FILE;
}
sub cat_into_file
{
        my $stuff = shift;
        my $file = shift;
        if ( -e "$file" ) {
                die "exists: $file";
        }
        open FILE, "> $file";
        print FILE $stuff;
        close FILE;
}
sub setufattr
{
	my ($file, $attr, $val) = @_;
	return undef if (! -e $file);
	if (!length($val)) {
		die "empty value for '$attr': '$val'\n";
	}
	my $ret = setfattr($file, 'user.'.$attr, $val);
	if (!defined $ret || !$ret) {
		printf STDERR "failed ($ret): setfattr('%s', '%s', '%s')\n", $file, 'user.'.$attr, $val;
		print STDERR $!."\n";
		return undef;
	}
	return 0;
}
sub delufattr
{
	my ($file, $attr) = @_;
	return undef if (! -e $file);
	delfattr($file, 'user.'.$attr);
}

sub getufattr
{
	my ($file, $attr) = @_;
	return undef if (! -e $file);
	getfattr($file, 'user.'.$attr);
}

sub set_dated_ufattr
{
	my ($file, $attr, $val) = @_;
        setufattr($file, $attr, $val);
        setufattr($file, $attr.'.mtime', time());
}

sub get_dated_ufattr
{
	my ($file, $attr, $val) = @_;
        my $attr_age = getufattr($file, $attr.".mtime");
	return undef if (!defined $attr_age);

	if (filemtime($file) > $attr_age) {
		delufattr($file, $attr);
        	delufattr($file, $attr.".mtime");
		return undef;
	}
        return getufattr($file, $attr);
}


my %md5_cache;
my $sum = new MD5; 
sub get_md5_of_string
{
	my $string = shift;
	$sum->reset();
        $sum->add($string);
        return $sum->hexdigest;
}
my %md5_cache;
my $md5_sum_dave = new MD5; 
sub __get_md5
{
	use MD5;
	my $file = shift;
	if (defined $md5_cache{$file}) {
		return $md5_cache{$file};
	}
	my $digest = getufattr($file, 'md5');
	if (defined $digest) {
		return $digest;
	}
	$md5_sum_dave->reset();
	if (!(-f $file && -r $file && open(FILE, "<$file"))) {
		print STDERR "__get_md5(): error opening '$file' for read\n";
		return undef;
	}
	my $fh = *FILE;
	$md5_sum_dave->addfile($fh);
        $digest = $md5_sum_dave->hexdigest;
	close FILE;
	setufattr($file, 'md5', $digest);
	$md5_cache{$file} = $digest;
	return $digest;
}
sub get_md5
{
	my $filename = shift;
	if ($filename =~ /\.md5cache/) {
		return undef;
	}
	my $md5 = __get_md5($filename);
	if ($md5) {
		cache_md5($filename,$md5);
	}
	return $md5;
}

sub samesum
{
	my $a = shift;
	my $b = shift;
	my $asum = get_md5($a);
	my $bsum = get_md5($b);
	if ($asum && $bsum && ($asum eq $bsum)) {
		return 1;
	}
	return 0;
}
sub same_md5_exists_here
{
	my $file = shift;
	my $dir = shift;
        my $link_name = $dir."/".get_md5_cache_path(__get_md5($file));
        check_cache_link($link_name);
        if (-e $link_name) {
		return readlink($link_name);
	}
	return undef;
}


sub min
{
	if ($_[0] < $_[1]) {
		return $_[0];
	}
	return $_[1];
}

sub max
{
	if ($_[0] > $_[1]) {
		return $_[0];
	}
	return $_[1];
}

sub samefile
{
	my ($f1, $f2) = @_;
	my ($f1dev,$f1ino) = stat($f1);
	my ($f2dev,$f2ino) = stat($f2);
	if ($f1dev == $f2dev && $f1ino == $f2ino) {
		return 1;
	}
	return 0;
}

sub filesize
{
	my ($file) = @_;
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
	    $atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
	return $size;
}

sub filemtime
{
	my ($file) = @_;
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
	    $atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
	return $mtime
}

sub file_ino_dev
{
	my ($file) = @_;
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
	    $atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
	return $ino.".".$dev;
}

sub __ls
{
        my $dir = shift;
        if (! -d $dir) {
                die "not a directory: '$dir'";
        }
        opendir (DIR, $dir) || die "Error opening directory $dir : $!\n";
        my @f = readdir(DIR);
        #map { $_ = $db_dir."/".$_ } @f;
        closedir (DIR) || die "Error closing directory $dir : $!\n";
        return @f
}

sub ls
{
	my @ret;
	foreach my $ent (@_) {
		push @ret, __ls($ent);
	}
	return @ret;
}

sub ls_nodot
{
	my @ret;
	foreach my $ent (ls(@_)) {
		next if special_dir($ent);
		push @ret, $ent;
	}
	return @ret;
}

sub special_dir
{
	my $dir = shift;
 	return 1 if ($dir eq "." || $dir eq "..");
	return 0;
}

sub findfiles
{
	my @dirs = @_;
	my @res;
	foreach my $dir (@dirs) {
		foreach my $dirent (ls $dir) {
			next if special_dir($dirent);
			my $fullname = $dir.'/'.$dirent;
			if (! -d $fullname) {
				push @res, $fullname;
			} else {
				push @res, findfiles($fullname);
			}
		}
	}
	return @res;
}

sub finddirs
{
	my @dirs = @_;
	my @res;
	foreach my $dir (@dirs) {
		foreach my $dirent (ls $dir) {
			next if ($dirent eq "." || $dirent eq "..");
			my $fullname = $dir.'/'.$dirent;
			if (! -d $fullname) {
				push @res, $fullname;
				push @res, findfiles($fullname);
			}
		}
	}
	return @res;
}

sub get_cached_dirsize
{
	my $dir = shift;
	my $cached_size = getufattr($dir, "du");
	return undef if ! defined $cached_size;
	my $cached_size_timestamp = getufattr($dir, "du.mtime");
	if ($cached_size_timestamp < filemtime($dir)) {
		return undef;
	}
	return $cached_size;
}

sub __du_dir
{
	my $du_visited_hash = shift;
	my $dir = shift;
	my $cached_res = get_cached_dirsize($dir);
	if (defined $cached_res) {
		return $cached_res;
	}
	my $start_mtime = time();

	use Cwd;
	my $olddir = fastgetcwd;
	my @entries = ls($dir);
	chdir $dir;
	my $res = __du($du_visited_hash, @entries);
	chdir $olddir;
	setufattr($dir, "du", $res);
	setufattr($dir, "du.mtime", $start_mtime);
	return $res;
}

sub __du
{
	my $du_visited_hash = shift;
	my @arg = @_;
	my $res = 0;
	foreach my $dirent (@arg) {
		next if ($dirent eq ".." || $dirent eq ".");
		if (! -d $dirent) {
			$res += filesize($dirent);
		} else {
			my $dir_ident = file_ino_dev($dirent);
			if (defined $du_visited_hash->{$dir_ident}) {
				printf "recursive directories: '%s' '%s'\n",
					$dirent, $du_visited_hash->{$dir_ident};
				next;
			}
			use Cwd;
			$du_visited_hash->{$dir_ident} = fastgetcwd()."/".$dirent;
			$res += __du_dir($du_visited_hash, $dirent);
		}
	}
	return $res;
}

sub du
{
	my %du_visited_hash;
	return __du(\%du_visited_hash, @_);
}

sub sizeprint
{
	my $size = shift;
	return sprintf "%7d KB", $size/1024;
	if ($size < 1<<10) {
		return sprintf "%3d B", $size;
	} elsif ($size < 1<<20) {
		return sprintf "%3d KB", $size>>10;
	} elsif ($size < 1<<30) {
		return sprintf "%3d MB", $size>>20;
	} 
	return sprintf "%3d GB", $size;
}
sub get_md5_cache_path
{
	my $md5 = shift;
	my $part1 = substr($md5,0,2);
	my $part2 = substr($md5,2,2);
	my $dir = sprintf ".md5cache/%s/%s", $part1, $part2;
 	my $link_name = $dir."/".$md5;
	return $link_name;
}

sub unique
{
	my @list = @_;
	my %tmphash;
	my @ret;
	for (my $i = 0; $i < scalar @list; $i++) {
		next if defined $tmphash{$list[$i]};
		$tmphash{$list[$i]} = 1;
		push @ret, $list[$i];
	}
	return @ret;
}

sub check_cache_link
{
	my $link_name = shift;
	my $target = readlink($link_name);
	if (-e $link_name && !defined $target) {
		unlink $link_name;
	}
}
sub cache_md5
{
	return;
	my $filename = shift;
	my $md5 = shift;
	my $link_name = dirpart($filename)."/".get_md5_cache_path($md5);
	check_cache_link($link_name);
	if (-e $link_name) {
		my $target = readlink($link_name);
		if (!samefile($target, $filename)) {
			use Carp;
			croak "duplicate md5 $filename $target $md5\n";
		}
	}
	mkdir_p(dirpart($link_name));
	symlink $filename,$link_name;
}
sub same_md5_exists_here
{
	my $file = shift;
	my $dir = shift;
        my $link_name = $dir."/".get_md5_cache_path(__get_md5($file));
        check_cache_link($link_name);
        if (-e $link_name) {
		return readlink($link_name);
	}
	return undef;
}

sub unique_rename
{	
	(my $orig, my $orig_new) = @_;
	my $new = $orig_new;
	my $collision_number = 0;
	while (-e $new) {
		$collision_number++;
		(my $fname,my $fpath) = fileparse($orig_new);
		$new = "$fpath/rename-collide.$collision_number.$fname";
	}
	my $bexist = (-e $new);
	my $boexist = (-e $orig);
	my $res = rename $orig, $new;
	if ($res < 0 || $res > 1) {
		my $exist = (-e $new);
		my $oexist = (-e $orig);
		print STDERR "rename $orig -> $new failed: $res exist: $exist:$bexist orig exist: $oexist:$boexist\n";
	}
	return $res;
}

sub mv_file
{
	my $file = shift;
	my $dir = shift;
	my $dest = $dir."/".basename($file);
	my $ret = unique_rename($file, $dest);
	die "mv_file error($ret): $!" if ($ret < 0);
}

my $sha1_sum_dave = new Digest::SHA1; 
my %sha1_cache;
sub get_sha1
{
	use Digest::SHA1;
	my $file = shift;
	if (defined $sha1_cache{$file}) {
		return $sha1_cache{$file};
	}
	my $digest = getufattr($file, 'sha1');
	if (defined $digest) {
		return $digest;
	}
	$sha1_sum_dave->reset();
	if (!(-f $file && -r $file && open(FILE, "<$file"))) {
		print STDERR "error opening $file for read\n";
		return undef;
	}
	my $fh = *FILE;
	$sha1_sum_dave->addfile($fh);
        $digest = $sha1_sum_dave->hexdigest;
	close FILE;
	setufattr($file, 'sha1', $digest);
	$sha1_cache{$file} = $digest;
	return $digest;
}

sub pfile
{
	my $format = shift;
	my $nr = shift;
	return sprintf "$format", $nr;
}
sub is_unique_file
{
	my $format = shift;
	my $nr = shift;
	return ! (-e pfile($format, $nr));
}
sub is_next_unique_file
{
	my $format = shift;
	my $nr = shift;
	if ($nr <= 0) {
		return is_unique_file($format, $nr);
	}
	return (is_unique_file($format, $nr) && 
		!is_unique_file($format, $nr-1)) 
}
sub uniquename_printf
{
	my $format = shift;;
	my $nr = 0;
	my $jump = 0;
	while (!is_unique_file($nr)) {
		$nr = (1<<$jump++);
	}
	$jump--;
	while (!is_next_unique_file($format, $nr)) {
		if (is_unique_file($format, $nr)) {
			$nr -= (1<<$jump);
		} else {
			$nr += (1<<$jump);
		}
		$jump--;
	}
	return pfile($format, $nr);
}

1;
